@@ -40,36 +40,35 @@
 #endif
 
 /**
- * SECTION:gapplicationcommandline
- * @title: GApplicationCommandLine
- * @short_description: A command-line invocation of an application
- * @include: gio/gio.h
- * @see_also: #GApplication
+ * GApplicationCommandLine:
  *
- * #GApplicationCommandLine represents a command-line invocation of
- * an application.  It is created by #GApplication and emitted
- * in the #GApplication::command-line signal and virtual function.
+ * `GApplicationCommandLine` represents a command-line invocation of
+ * an application.
+ *
+ * It is created by [class@Gio.Application] and emitted
+ * in the [signal@Gio.Application::command-line] signal and virtual function.
  *
  * The class contains the list of arguments that the program was invoked
- * with.  It is also possible to query if the commandline invocation was
+ * with. It is also possible to query if the commandline invocation was
  * local (ie: the current process is running in direct response to the
  * invocation) or remote (ie: some other process forwarded the
  * commandline to this process).
  *
- * The GApplicationCommandLine object can provide the @argc and @argv
- * parameters for use with the #GOptionContext command-line parsing API,
- * with the g_application_command_line_get_arguments() function. See
+ * The `GApplicationCommandLine` object can provide the @argc and @argv
+ * parameters for use with the [struct@GLib.OptionContext] command-line parsing API,
+ * with the [method@Gio.ApplicationCommandLine.get_arguments] function. See
  * [gapplication-example-cmdline3.c][gapplication-example-cmdline3]
  * for an example.
  *
  * The exit status of the originally-invoked process may be set and
- * messages can be printed to stdout or stderr of that process.  The
- * lifecycle of the originally-invoked process is tied to the lifecycle
- * of this object (ie: the process exits when the last reference is
- * dropped).
+ * messages can be printed to stdout or stderr of that process.
  *
- * The main use for #GApplicationCommandLine (and the
- * #GApplication::command-line signal) is 'Emacs server' like use cases:
+ * For remote invocation, the originally-invoked process exits when
+ * [method@Gio.ApplicationCommandLine.done] method is called. This method is
+ * also automatically called when the object is disposed.
+ *
+ * The main use for `GApplicationCommandLine` (and the
+ * [signal@Gio.Application::command-line] signal) is 'Emacs server' like use cases:
  * You can set the `EDITOR` environment variable to have e.g. git use
  * your favourite editor to edit commit messages, and if you already
  * have an instance of the editor running, the editing will happen
@@ -78,11 +77,12 @@
  * does not return until the editing is done.
  *
  * Normally, the commandline is completely handled in the
- * #GApplication::command-line handler. The launching instance exits
+ * [signal@Gio.Application::command-line] handler. The launching instance exits
  * once the signal handler in the primary instance has returned, and
  * the return value of the signal handler becomes the exit status
  * of the launching instance.
- * |[<!-- language="C" -->
+ *
+ * ```c
  * static int
  * command_line (GApplication            *application,
  *               GApplicationCommandLine *cmdline)
@@ -104,13 +104,15 @@
  *
  *   return 0;
  * }
- * ]|
- * The complete example can be found here: 
+ * ```
+ *
+ * The complete example can be found here:
  * [gapplication-example-cmdline.c](https://gitlab.gnome.org/GNOME/glib/-/blob/HEAD/gio/tests/gapplication-example-cmdline.c)
  *
  * In more complicated cases, the handling of the commandline can be
  * split between the launcher and the primary instance.
- * |[<!-- language="C" -->
+ *
+ * ```c
  * static gboolean
  *  test_local_cmdline (GApplication   *application,
  *                      gchar        ***arguments,
@@ -156,18 +158,19 @@
  *
  *   ...
  * }
- * ]|
+ * ```
+ *
  * In this example of split commandline handling, options that start
  * with `--local-` are handled locally, all other options are passed
- * to the #GApplication::command-line handler which runs in the primary
+ * to the [signal@Gio.Application::command-line] handler which runs in the primary
  * instance.
  *
  * The complete example can be found here:
  * [gapplication-example-cmdline2.c](https://gitlab.gnome.org/GNOME/glib/-/blob/HEAD/gio/tests/gapplication-example-cmdline2.c)
  *
- * If handling the commandline requires a lot of work, it may
- * be better to defer it.
- * |[<!-- language="C" -->
+ * If handling the commandline requires a lot of work, it may be better to defer it.
+ *
+ * ```c
  * static gboolean
  * my_cmdline_handler (gpointer data)
  * {
@@ -197,10 +200,11 @@
  *
  *   return 0;
  * }
- * ]|
+ * ```
+ *
  * In this example the commandline is not completely handled before
- * the #GApplication::command-line handler returns. Instead, we keep
- * a reference to the #GApplicationCommandLine object and handle it
+ * the [signal@Gio.Application::command-line] handler returns. Instead, we keep
+ * a reference to the `GApplicationCommandLine` object and handle it
  * later (in this example, in an idle). Note that it is necessary to
  * hold the application until you are done with the commandline.
  *
@@ -208,13 +212,6 @@
  * [gapplication-example-cmdline3.c](https://gitlab.gnome.org/GNOME/glib/-/blob/HEAD/gio/tests/gapplication-example-cmdline3.c)
  */
 
-/**
- * GApplicationCommandLine:
- *
- * #GApplicationCommandLine is an opaque data structure and can only be accessed
- * using the following functions.
- */
-
 /**
  * GApplicationCommandLineClass:
  *
@@ -242,6 +239,7 @@ struct _GApplicationCommandLinePrivate
 
   gchar **environ;
   gint exit_status;
+  gboolean done;
 };
 
 G_DEFINE_TYPE_WITH_PRIVATE (GApplicationCommandLine, g_application_command_line, G_TYPE_OBJECT)
@@ -260,20 +258,20 @@ grok_platform_data (GApplicationCommandLine *cmdline)
   g_variant_iter_init (&iter, cmdline->priv->platform_data);
 
   while (g_variant_iter_loop (&iter, "{&sv}", &key, &value))
-    if (strcmp (key, "cwd") == 0)
+    if (strcmp (key, "cwd") == 0 && g_variant_is_of_type (value, G_VARIANT_TYPE_BYTESTRING))
       {
         if (!cmdline->priv->cwd)
           cmdline->priv->cwd = g_variant_dup_bytestring (value, NULL);
       }
 
-    else if (strcmp (key, "environ") == 0)
+    else if (strcmp (key, "environ") == 0 && g_variant_is_of_type (value, G_VARIANT_TYPE_BYTESTRING_ARRAY))
       {
         if (!cmdline->priv->environ)
           cmdline->priv->environ =
             g_variant_dup_bytestring_array (value, NULL);
       }
 
-    else if (strcmp (key, "options") == 0)
+    else if (strcmp (key, "options") == 0 && g_variant_is_of_type (value, G_VARIANT_TYPE_VARDICT))
       {
         if (!cmdline->priv->options)
           cmdline->priv->options = g_variant_ref (value);
@@ -304,6 +302,11 @@ g_application_command_line_real_get_stdin (GApplicationCommandLine *cmdline)
 #endif
 }
 
+static void
+g_application_command_line_real_done (GApplicationCommandLine *cmdline)
+{
+}
+
 static void
 g_application_command_line_get_property (GObject    *object,
                                          guint       prop_id,
@@ -363,6 +366,16 @@ g_application_command_line_set_property (GObject      *object,
     }
 }
 
+static void
+g_application_command_line_dispose (GObject *object)
+{
+  GApplicationCommandLine *cmdline = G_APPLICATION_COMMAND_LINE (object);
+
+  g_application_command_line_done (cmdline);
+
+  G_OBJECT_CLASS (g_application_command_line_parent_class)->dispose (object);
+}
+
 static void
 g_application_command_line_finalize (GObject *object)
 {
@@ -416,39 +429,63 @@ g_application_command_line_class_init (GApplicationCommandLineClass *class)
   object_class->get_property = g_application_command_line_get_property;
   object_class->set_property = g_application_command_line_set_property;
   object_class->finalize = g_application_command_line_finalize;
+  object_class->dispose = g_application_command_line_dispose;
   object_class->constructed = g_application_command_line_constructed;
 
   class->printerr_literal = g_application_command_line_real_printerr_literal;
   class->print_literal = g_application_command_line_real_print_literal;
   class->get_stdin = g_application_command_line_real_get_stdin;
 
+  class->done = g_application_command_line_real_done;
+
+  /**
+   * GApplicationCommandLine:arguments:
+   *
+   * The commandline that caused this [signal@Gio.Application::command-line]
+   * signal emission.
+   *
+   * Since: 2.28
+   */
   g_object_class_install_property (object_class, PROP_ARGUMENTS,
-    g_param_spec_variant ("arguments",
-                          P_("Commandline arguments"),
-                          P_("The commandline that caused this ::command-line signal emission"),
+    g_param_spec_variant ("arguments", NULL, NULL,
                           G_VARIANT_TYPE_BYTESTRING_ARRAY, NULL,
                           G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY |
                           G_PARAM_STATIC_STRINGS));
 
+  /**
+   * GApplicationCommandLine:options:
+   *
+   * The options sent along with the commandline.
+   *
+   * Since: 2.28
+   */
   g_object_class_install_property (object_class, PROP_OPTIONS,
-    g_param_spec_variant ("options",
-                          P_("Options"),
-                          P_("The options sent along with the commandline"),
+    g_param_spec_variant ("options", NULL, NULL,
                           G_VARIANT_TYPE_VARDICT, NULL, G_PARAM_WRITABLE |
                           G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS));
 
+  /**
+   * GApplicationCommandLine:platform-data:
+   *
+   * Platform-specific data for the commandline.
+   *
+   * Since: 2.28
+   */
   g_object_class_install_property (object_class, PROP_PLATFORM_DATA,
-    g_param_spec_variant ("platform-data",
-                          P_("Platform data"),
-                          P_("Platform-specific data for the commandline"),
+    g_param_spec_variant ("platform-data", NULL, NULL,
                           G_VARIANT_TYPE ("a{sv}"), NULL,
                           G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY |
                           G_PARAM_STATIC_STRINGS));
 
+  /**
+   * GApplicationCommandLine:is-remote:
+   *
+   * Whether this is a remote commandline.
+   *
+   * Since: 2.28
+   */
   g_object_class_install_property (object_class, PROP_IS_REMOTE,
-    g_param_spec_boolean ("is-remote",
-                          P_("Is remote"),
-                          P_("TRUE if this is a remote commandline"),
+    g_param_spec_boolean ("is-remote", NULL, NULL,
                           FALSE,
                           G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 }
@@ -497,7 +534,7 @@ g_application_command_line_get_arguments (GApplicationCommandLine *cmdline,
  * g_application_command_line_get_options_dict:
  * @cmdline: a #GApplicationCommandLine
  *
- * Gets the options there were passed to g_application_command_line().
+ * Gets the options that were passed to g_application_command_line().
  *
  * If you did not override local_command_line() then these are the same
  * options that were parsed according to the #GOptionEntrys added to the
@@ -507,6 +544,9 @@ g_application_command_line_get_arguments (GApplicationCommandLine *cmdline,
  * If no options were sent then an empty dictionary is returned so that
  * you don't need to check for %NULL.
  *
+ * The data has been passed via an untrusted external process, so the types of
+ * all values must be checked before being used.
+ *
  * Returns: (transfer none): a #GVariantDict with the options
  *
  * Since: 2.40
@@ -655,6 +695,54 @@ g_application_command_line_get_is_remote (GApplicationCommandLine *cmdline)
   return IS_REMOTE (cmdline);
 }
 
+/**
+ * g_application_command_line_print_literal:
+ * @cmdline: a #GApplicationCommandLine
+ * @message: the message
+ *
+ * Prints a message using the stdout print handler in the invoking process.
+ *
+ * Unlike g_application_command_line_print(), @message is not a `printf()`-style
+ * format string. Use this function if @message contains text you don't have
+ * control over, that could include `printf()` escape sequences.
+ *
+ * Since: 2.80
+ **/
+void
+g_application_command_line_print_literal (GApplicationCommandLine *cmdline,
+                                          const gchar             *message)
+{
+  g_return_if_fail (G_IS_APPLICATION_COMMAND_LINE (cmdline));
+  g_return_if_fail (message != NULL);
+
+  G_APPLICATION_COMMAND_LINE_GET_CLASS (cmdline)
+    ->print_literal (cmdline, message);
+}
+
+/**
+ * g_application_command_line_printerr_literal:
+ * @cmdline: a #GApplicationCommandLine
+ * @message: the message
+ *
+ * Prints a message using the stderr print handler in the invoking process.
+ *
+ * Unlike g_application_command_line_printerr(), @message is not
+ * a `printf()`-style format string. Use this function if @message contains text
+ * you don't have control over, that could include `printf()` escape sequences.
+ *
+ * Since: 2.80
+ **/
+void
+g_application_command_line_printerr_literal (GApplicationCommandLine *cmdline,
+                                             const gchar             *message)
+{
+  g_return_if_fail (G_IS_APPLICATION_COMMAND_LINE (cmdline));
+  g_return_if_fail (message != NULL);
+
+  G_APPLICATION_COMMAND_LINE_GET_CLASS (cmdline)
+    ->printerr_literal (cmdline, message);
+}
+
 /**
  * g_application_command_line_print:
  * @cmdline: a #GApplicationCommandLine
@@ -752,6 +840,9 @@ g_application_command_line_printerr (GApplicationCommandLine *cmdline,
  * always zero.  If the application use count is zero, though, the exit
  * status of the local #GApplicationCommandLine is used.
  *
+ * This method is a no-op if g_application_command_line_done() has
+ * been called.
+ *
  * Since: 2.28
  **/
 void
@@ -760,6 +851,9 @@ g_application_command_line_set_exit_status (GApplicationCommandLine *cmdline,
 {
   g_return_if_fail (G_IS_APPLICATION_COMMAND_LINE (cmdline));
 
+  if (cmdline->priv->done)
+    return;
+
   cmdline->priv->exit_status = exit_status;
 }
 
@@ -793,9 +887,12 @@ g_application_command_line_get_exit_status (GApplicationCommandLine *cmdline)
  * information like the current working directory and the startup
  * notification ID.
  *
+ * It comes from an untrusted external process and hence the types of all
+ * values must be validated before being used.
+ *
  * For local invocation, it will be %NULL.
  *
- * Returns: (nullable): the platform data, or %NULL
+ * Returns: (nullable) (transfer full): the platform data, or %NULL
  *
  * Since: 2.28
  **/
@@ -840,3 +937,38 @@ g_application_command_line_create_file_for_arg (GApplicationCommandLine *cmdline
 
   return g_file_new_for_commandline_arg (arg);
 }
+
+/**
+ * g_application_command_line_done:
+ * @cmdline: a #GApplicationCommandLine
+ *
+ * Signals that command line processing is completed.
+ *
+ * For remote invocation, it causes the invoking process to terminate.
+ *
+ * For local invocation, it does nothing.
+ *
+ * This method should be called in the [signal@Gio.Application::command-line]
+ * handler, after the exit status is set and all messages are printed.
+ *
+ * After this call, g_application_command_line_set_exit_status() has no effect.
+ * Subsequent calls to this method are no-ops.
+ *
+ * This method is automatically called when the #GApplicationCommandLine
+ * object is disposed — so you can omit the call in non-garbage collected
+ * languages.
+ *
+ * Since: 2.80
+ **/
+void
+g_application_command_line_done (GApplicationCommandLine *cmdline)
+{
+  g_return_if_fail (G_IS_APPLICATION_COMMAND_LINE (cmdline));
+
+  if (cmdline->priv->done)
+    return;
+
+  G_APPLICATION_COMMAND_LINE_GET_CLASS (cmdline)->done (cmdline);
+
+  cmdline->priv->done = TRUE;
+}
