@@ -33,17 +33,14 @@
 
 
 /**
- * SECTION:gconverterinputstream
- * @short_description: Converter Input Stream
- * @include: gio/gio.h
- * @see_also: #GInputStream, #GConverter
+ * GConverterInputStream:
  *
- * Converter input stream implements #GInputStream and allows
+ * Converter input stream implements [class@Gio.InputStream] and allows
  * conversion of data of various types during reading.
  *
- * As of GLib 2.34, #GConverterInputStream implements
- * #GPollableInputStream.
- **/
+ * As of GLib 2.34, `GConverterInputStream` implements
+ * [iface@Gio.PollableInputStream].
+ */
 
 #define INITIAL_BUFFER_SIZE 4096
 
@@ -116,11 +113,14 @@ g_converter_input_stream_class_init (GConverterInputStreamClass *klass)
   istream_class = G_INPUT_STREAM_CLASS (klass);
   istream_class->read_fn = g_converter_input_stream_read;
 
+  /**
+   * GConverterInputStream:converter:
+   *
+   * The converter object.
+   */
   g_object_class_install_property (object_class,
 				   PROP_CONVERTER,
-				   g_param_spec_object ("converter",
-							P_("Converter"),
-							P_("The converter object"),
+				   g_param_spec_object ("converter", NULL, NULL,
 							G_TYPE_CONVERTER,
 							G_PARAM_READWRITE|
 							G_PARAM_CONSTRUCT_ONLY|
