@@ -36,39 +36,9 @@
 #include "gtestutils.h"
 #include "gslice.h"
 
-/**
- * SECTION:trees-binary
- * @title: Balanced Binary Trees
- * @short_description: a sorted collection of key/value pairs optimized
- *                     for searching and traversing in order
- *
- * The #GTree structure and its associated functions provide a sorted
- * collection of key/value pairs optimized for searching and traversing
- * in order. This means that most of the operations  (access, search,
- * insertion, deletion, ...) on #GTree are O(log(n)) in average and O(n)
- * in worst case for time complexity. But, note that maintaining a
- * balanced sorted #GTree of n elements is done in time O(n log(n)).
- *
- * To create a new #GTree use g_tree_new().
- *
- * To insert a key/value pair into a #GTree use g_tree_insert()
- * (O(n log(n))).
- *
- * To remove a key/value pair use g_tree_remove() (O(n log(n))).
- *
- * To look up the value corresponding to a given key, use
- * g_tree_lookup() and g_tree_lookup_extended().
- *
- * To find out the number of nodes in a #GTree, use g_tree_nnodes(). To
- * get the height of a #GTree, use g_tree_height().
- *
- * To traverse a #GTree, calling a function for each node visited in
- * the traversal, use g_tree_foreach().
- *
- * To destroy a #GTree, use g_tree_destroy().
- **/
-
 #define MAX_GTREE_HEIGHT 40
+/* G_MAXUINT nodes will be covered by tree height of log2(G_MAXUINT) + 2. */
+G_STATIC_ASSERT ((G_GUINT64_CONSTANT (1) << (MAX_GTREE_HEIGHT - 2)) >= G_MAXUINT);
 
 /**
  * GTree:
@@ -105,7 +75,8 @@ static GTreeNode* g_tree_node_new                   (gpointer       key,
 static GTreeNode *g_tree_insert_internal (GTree *tree,
                                           gpointer key,
                                           gpointer value,
-                                          gboolean replace);
+                                          gboolean replace,
+                                          gboolean null_ret_ok);
 static gboolean   g_tree_remove_internal            (GTree         *tree,
                                                      gconstpointer  key,
                                                      gboolean       steal);
@@ -149,7 +120,7 @@ g_tree_node_new (gpointer key,
 }
 
 /**
- * g_tree_new:
+ * g_tree_new: (constructor)
  * @key_compare_func: the function used to order the nodes in the #GTree.
  *   It should return values similar to the standard strcmp() function -
  *   0 if the two arguments are equal, a negative value if the first argument 
@@ -454,6 +425,26 @@ g_tree_destroy (GTree *tree)
   g_tree_unref (tree);
 }
 
+static GTreeNode *
+g_tree_insert_replace_node_internal (GTree *tree,
+                                     gpointer key,
+                                     gpointer value,
+                                     gboolean replace,
+                                     gboolean null_ret_ok)
+{
+  GTreeNode *node;
+
+  g_return_val_if_fail (tree != NULL, NULL);
+
+  node = g_tree_insert_internal (tree, key, value, replace, null_ret_ok);
+
+#ifdef G_TREE_DEBUG
+  g_tree_node_check (tree->root);
+#endif
+
+  return node;
+}
+
 /**
  * g_tree_insert_node:
  * @tree: a #GTree
@@ -474,7 +465,8 @@ g_tree_destroy (GTree *tree)
  * result in a O(n log(n)) operation where most of the other operations
  * are O(log(n)).
  *
- * Returns: (transfer none): the inserted (or set) node.
+ * Returns: (transfer none) (nullable): the inserted (or set) node or %NULL
+ * if insertion would overflow the tree node counter.
  *
  * Since: 2.68
  */
@@ -483,17 +475,7 @@ g_tree_insert_node (GTree    *tree,
                     gpointer  key,
                     gpointer  value)
 {
-  GTreeNode *node;
-
-  g_return_val_if_fail (tree != NULL, NULL);
-
-  node = g_tree_insert_internal (tree, key, value, FALSE);
-
-#ifdef G_TREE_DEBUG
-  g_tree_node_check (tree->root);
-#endif
-
-  return node;
+  return g_tree_insert_replace_node_internal (tree, key, value, FALSE, TRUE);
 }
 
 /**
@@ -512,7 +494,7 @@ g_tree_insert (GTree    *tree,
                gpointer  key,
                gpointer  value)
 {
-  g_tree_insert_node (tree, key, value);
+  g_tree_insert_replace_node_internal (tree, key, value, FALSE, FALSE);
 }
 
 /**
@@ -531,7 +513,8 @@ g_tree_insert (GTree    *tree,
  * The tree is automatically 'balanced' as new key/value pairs are added,
  * so that the distance from the root to every leaf is as small as possible.
  *
- * Returns: (transfer none): the inserted (or set) node.
+ * Returns: (transfer none) (nullable): the inserted (or set) node or %NULL
+ * if insertion would overflow the tree node counter.
  *
  * Since: 2.68
  */
@@ -540,17 +523,7 @@ g_tree_replace_node (GTree    *tree,
                      gpointer  key,
                      gpointer  value)
 {
-  GTreeNode *node;
-
-  g_return_val_if_fail (tree != NULL, NULL);
-
-  node = g_tree_insert_internal (tree, key, value, TRUE);
-
-#ifdef G_TREE_DEBUG
-  g_tree_node_check (tree->root);
-#endif
-
-  return node;
+  return g_tree_insert_replace_node_internal (tree, key, value, TRUE, TRUE);
 }
 
 /**
@@ -567,7 +540,26 @@ g_tree_replace (GTree    *tree,
                 gpointer  key,
                 gpointer  value)
 {
-  g_tree_replace_node (tree, key, value);
+  g_tree_insert_replace_node_internal (tree, key, value, TRUE, FALSE);
+}
+
+/* internal checked nnodes increment routine */
+static gboolean
+g_tree_nnodes_inc_checked (GTree *tree, gboolean overflow_fatal)
+{
+  if (G_UNLIKELY (tree->nnodes == G_MAXUINT))
+    {
+      if (overflow_fatal)
+        {
+          g_error ("Incrementing GTree nnodes counter would overflow");
+        }
+
+      return FALSE;
+    }
+
+  tree->nnodes++;
+
+  return TRUE;
 }
 
 /* internal insert routine */
@@ -575,7 +567,8 @@ static GTreeNode *
 g_tree_insert_internal (GTree    *tree,
                         gpointer  key,
                         gpointer  value,
-                        gboolean  replace)
+                        gboolean  replace,
+                        gboolean  null_ret_ok)
 {
   GTreeNode *node, *retnode;
   GTreeNode *path[MAX_GTREE_HEIGHT];
@@ -586,7 +579,12 @@ g_tree_insert_internal (GTree    *tree,
   if (!tree->root)
     {
       tree->root = g_tree_node_new (key, value);
+
+#ifdef G_TREE_DEBUG
+      g_assert (tree->nnodes == 0);
+#endif
       tree->nnodes++;
+
       return tree->root;
     }
 
@@ -630,16 +628,20 @@ g_tree_insert_internal (GTree    *tree,
             }
           else
             {
-              GTreeNode *child = g_tree_node_new (key, value);
+              GTreeNode *child;
 
+              if (!g_tree_nnodes_inc_checked (tree, !null_ret_ok))
+                {
+                  return NULL;
+                }
+
+              child = g_tree_node_new (key, value);
               child->left = node->left;
               child->right = node;
               node->left = child;
               node->left_child = TRUE;
               node->balance -= 1;
 
-              tree->nnodes++;
-
               retnode = child;
               break;
             }
@@ -653,16 +655,20 @@ g_tree_insert_internal (GTree    *tree,
             }
           else
             {
-              GTreeNode *child = g_tree_node_new (key, value);
+              GTreeNode *child;
 
+              if (!g_tree_nnodes_inc_checked (tree, !null_ret_ok))
+                {
+                  return NULL;
+                }
+
+              child = g_tree_node_new (key, value);
               child->right = node->right;
               child->left = node;
               node->right = child;
               node->right_child = TRUE;
               node->balance += 1;
 
-              tree->nnodes++;
-
               retnode = child;
               break;
             }
@@ -1100,7 +1106,7 @@ g_tree_lookup_extended (GTree         *tree,
 /**
  * g_tree_foreach:
  * @tree: a #GTree
- * @func: the function to call for each node visited.
+ * @func: (scope call): the function to call for each node visited.
  *     If this function returns %TRUE, the traversal is stopped.
  * @user_data: user data to pass to the function
  *
@@ -1139,7 +1145,7 @@ g_tree_foreach (GTree         *tree,
 /**
  * g_tree_foreach_node:
  * @tree: a #GTree
- * @func: the function to call for each node visited.
+ * @func: (scope call): the function to call for each node visited.
  *     If this function returns %TRUE, the traversal is stopped.
  * @user_data: user data to pass to the function
  *
@@ -1180,7 +1186,7 @@ g_tree_foreach_node (GTree             *tree,
 /**
  * g_tree_traverse:
  * @tree: a #GTree
- * @traverse_func: the function to call for each node visited. If this 
+ * @traverse_func: (scope call): the function to call for each node visited. If this
  *   function returns %TRUE, the traversal is stopped.
  * @traverse_type: the order in which nodes are visited, one of %G_IN_ORDER,
  *   %G_PRE_ORDER and %G_POST_ORDER
@@ -1197,7 +1203,7 @@ g_tree_foreach_node (GTree             *tree,
  * GTraverseFunc:
  * @key: a key of a #GTree node
  * @value: the value corresponding to the key
- * @user_data: user data passed to g_tree_traverse()
+ * @data: user data passed to g_tree_traverse()
  *
  * Specifies the type of function passed to g_tree_traverse(). It is
  * passed the key and value of each node, together with the @user_data
@@ -1240,7 +1246,7 @@ g_tree_traverse (GTree         *tree,
 /**
  * g_tree_search_node:
  * @tree: a #GTree
- * @search_func: a function used to search the #GTree
+ * @search_func: (scope call): a function used to search the #GTree
  * @user_data: the data passed as the second argument to @search_func
  *
  * Searches a #GTree using @search_func.
@@ -1274,7 +1280,7 @@ g_tree_search_node (GTree         *tree,
 /**
  * g_tree_search:
  * @tree: a #GTree
- * @search_func: a function used to search the #GTree
+ * @search_func: (scope call): a function used to search the #GTree
  * @user_data: the data passed as the second argument to @search_func
  *
  * Searches a #GTree using @search_func.
@@ -1454,6 +1460,11 @@ g_tree_height (GTree *tree)
  * Gets the number of nodes in a #GTree.
  * 
  * Returns: the number of nodes in @tree
+ *
+ * The node counter value type is really a #guint,
+ * but it is returned as a #gint due to backward
+ * compatibility issues (can be cast back to #guint to
+ * support its full range of values).
  */
 gint
 g_tree_nnodes (GTree *tree)
